@@ -39,6 +39,7 @@ class Marginalia(NamedTuple):
 
 class Field(NamedTuple):
 	name:Name
+	zone:Name
 	shape:object # Marginalia -> Leaf; Name -> named shape; Frame/Menu/Tree: define as such.
 
 class Frame(NamedTuple):
@@ -67,3 +68,17 @@ class Canvas(NamedTuple):
 	style_points:list
 	patches:list
 
+class Criterion(NamedTuple):
+	field_name: Name
+	predicate: formulae.Predicate
+
+class Patch(NamedTuple):
+	is_merge: bool
+	criteria: List[Criterion]
+	content: object
+	style_points: list
+
+class PatchBlock(NamedTuple):
+	criteria: List[Criterion]
+	sub_patches: List[Union[Patch, "PatchBlock"]]
+
