@@ -201,6 +201,10 @@ class Canvas:
 			return [utility.make_range(c, r) for c in columns for r in rows]
 		else:
 			return ["0"]
+	
+	def zone(self, key) -> Dict[str,str]:
+		""" DTSTTCPW dictates this means of exposing data zones to the application. """
+		return self.definition.zones[key]
 
 class FormulaInterpreter(foundation.Visitor):
 	"""
@@ -410,8 +414,7 @@ class LeafTour(foundation.Visitor):
 class InternalTour(NodeFilter):
 	"""
 	Yield matching (internal, if possible) nodes.
-	This is useful for merges, for outline specifications,
-	and possibly for various other activities.
+	This is useful for merges and possibly later for enumerating named ranges.
 	"""
 	
 	def __init__(self, cursor: dict, selection: formulae.Selection):
