@@ -6,7 +6,7 @@ from . import AST, formulae, utility
 TABLES = utility.tables(__file__, 'core.md')
 
 class CoreDriver(brt.TypicalApplication):
-	VALID_KEYWORDS = {'AXIS', 'CANVAS', 'FRAME', 'GAP', 'HEAD', 'LEAF', 'MENU', 'MERGE', 'OF', 'USE', 'TREE', 'STYLE'}
+	VALID_KEYWORDS = {'AXIS', 'CANVAS', 'FRAME', 'GAP', 'HEAD', 'LEAF', 'MENU', 'MERGE', 'STYLE', 'TREE', 'USE', 'ZONE'}
 	
 	def default_scan_action(self, message, scanner, param):
 		# Just in case I forgot something:
@@ -107,10 +107,14 @@ class CoreDriver(brt.TypicalApplication):
 	def parse_marginalia(self, texts, hint, appearance):
 		return AST.Marginalia(texts, hint, appearance)
 	
-	def parse_field(self, name, shape):
+	def parse_define_shape(self, name, shape):
+		return self.parse_field(name, None, shape)
+	
+	def parse_field(self, name, zone, shape):
 		assert isinstance(name, AST.Name), type(name)
+		assert zone is None or isinstance(zone, AST.Name), type(zone)
 		assert isinstance(shape, (AST.Marginalia, AST.Frame, AST.Menu, AST.Tree, AST.LinkRef)), type(shape)
-		return AST.Field(name, shape)
+		return AST.Field(name, zone, shape)
 	
 	def parse_frame(self, margin, key, fields): return AST.Frame(margin, key, fields)
 	def parse_menu(self, margin, key, fields): return AST.Menu(margin, key, fields)
@@ -122,8 +126,17 @@ class CoreDriver(brt.TypicalApplication):
 		assert isinstance(value, AST.Constant)
 		return AST.Assign(name, value)
 	
-	def parse_define_canvas(self, name:AST.Name, across:AST.Name, down:AST.Name, style_points:list, items:list):
-		return AST.Canvas(name, across, down, style_points, items)
+	def parse_define_canvas(self, name:AST.Name, across:AST.Name, down:AST.Name, style_points:list, patches:list):
+		return AST.Canvas(name, across, down, style_points, patches)
+	
+	def parse_patch_plain(self, criteria:List[AST.Criterion], content, style_points:list):
+		return AST.Patch(False, criteria, content, style_points)
+	
+	def parse_patch_merge(self, criteria:List[AST.Criterion], content, style_points:list):
+		return AST.Patch(True, criteria, content, style_points)
+	
+	def parse_patch_block(self, criteria:List[AST.Criterion], sub_patches:list):
+		return AST.PatchBlock(criteria, sub_patches)
 	
 	def parse_label_interpolated(self, items) -> formulae.Boilerplate:
 		if len(items) == 1 and isinstance(items[0], formulae.RawOrdinal):
@@ -154,18 +167,8 @@ class CoreDriver(brt.TypicalApplication):
 	def parse_select_each(self) -> formulae.IsDefined:
 		return formulae.IsDefined()
 	def parse_criterion(self, field_name:AST.Name, predicate:formulae.Predicate):
-		# TODO: You can make the argument that a name on the left-hand side of
-		#  a criterion ought to appear in any canvas that uses it. And sure,
-		#  that would be nice to validate. But not right this minute.
-		return field_name.text, predicate
-	def parse_selector(self, criteria:List[Tuple[str, formulae.Predicate]]):
-		cd = {}
-		for axis, predicate in criteria:
-			if axis not in cd: cd[axis] = predicate
-			elif isinstance(cd[axis], list): cd[axis].append(predicate)
-			else: cd[axis] = [cd[axis], predicate]
-		return formulae.Selection(cd)
-	def parse_formula(self, fragments): return formulae.Formula(fragments)
+		return AST.Criterion(field_name, predicate)
 	def parse_gap_hint(self): return AST.GAP_HINT
+	def parse_blank_cell(self): return formulae.THE_NOTHING
 	def parse_linkref(self, marginalia:AST.Marginalia, name:AST.Name):
 		return AST.LinkRef(marginalia, name)
